@@ -16,12 +16,12 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_REGISTRY_H
-#define UNITY_SCOPES_REGISTRY_H
+#pragma once
 
 #include <core/signal.h>
 #include <unity/scopes/Object.h>
 #include <unity/scopes/RegistryProxyFwd.h>
+#include <unity/scopes/Scope.h>
 #include <unity/scopes/ScopeMetadata.h>
 
 #include <map>
@@ -111,5 +111,3 @@ protected:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
