@@ -25,11 +25,7 @@
 #include <unity/util/NonCopyable.h>
 #include <unity/util/ResourcePtr.h>
 
-#include <sys/inotify.h>
-#include <sys/ioctl.h>
-
-#include <atomic>
-#include <thread>
+#include <time.h>
 
 namespace unity
 {
@@ -61,7 +57,7 @@ public:
                                    unity::scopes::internal::SettingsSchema const& schema,
                                    boost::log::sources::severity_channel_logger_mt<>& logger);
 
-    ~SettingsDB();
+    ~SettingsDB() = default;
 
     SettingsDB(SettingsDB&&) = default;
     SettingsDB& operator=(SettingsDB&&) = default;
@@ -69,14 +65,6 @@ public:
     VariantMap settings();  // Returns the current settings (checking the DB each time).
 
 private:
-    enum ThreadState
-    {
-        Idle,
-        Running,
-        Stopping,
-        Failed
-    };
-
     SettingsDB(std::string const& db_path,
                unity::scopes::internal::SettingsSchema const& schema,
                boost::log::sources::severity_channel_logger_mt<>& logger);
@@ -84,18 +72,14 @@ private:
     void process_doc_(std::string const& id, unity::util::IniParser const& parer);
     void process_all_docs();
     void set_defaults();
-    void watch_thread();
 
-    bool state_changed_;
     std::string db_path_;
-    unity::util::ResourcePtr<int, std::function<void(int)>> fd_;
-    unity::util::ResourcePtr<int, std::function<void(int)>> watch_;
+    decltype(::timespec::tv_nsec) last_write_time_nsec_;
+    decltype(::timespec::tv_sec) last_write_time_sec_;
+    ::ino_t last_write_inode_;
     VariantArray definitions_;                       // Returned by SettingsSchema
     std::map<std::string, Variant> def_map_;  // Allows fast access to the Variants in definitions_
     unity::scopes::VariantMap values_;
-    std::thread thread_;
-    std::mutex mutex_;
-    ThreadState thread_state_;
     boost::log::sources::severity_channel_logger_mt<>& logger_;
 };
 
