@@ -16,13 +16,14 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_ZMQMIDDLEWARE_OBJECTADAPTER_H
-#define UNITY_SCOPES_INTERNAL_ZMQMIDDLEWARE_OBJECTADAPTER_H
+#pragma once
 
+#include <unity/scopes/internal/Logger.h>
+#include <unity/scopes/internal/zmq_middleware/Current.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqObjectProxy.h>
-
 #include <unity/scopes/ScopeExceptions.h>
 #include <unity/util/NonCopyable.h>
+
 #include <zmqpp/socket.hpp>
 
 #include <future>
@@ -108,6 +109,12 @@ private:
 
     void store_exception(scopes::MiddlewareException& ex);
 
+    boost::log::sources::severity_channel_logger_mt<>& logger() const;
+    boost::log::sources::severity_channel_logger_mt<>& ipc_logger() const;
+
+    void trace_dispatch(Current const& c);
+    void trace_response();
+
     ZmqMiddleware& mw_;
     std::string name_;
     std::string endpoint_;
@@ -138,5 +145,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
