@@ -19,10 +19,13 @@
 
 #include <unity/scopes/internal/FilterBaseImpl.h>
 #include <unity/scopes/internal/FilterStateImpl.h>
+#include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/smartscopes/SmartScopesClient.h>
+#include <unity/scopes/internal/Utils.h>
 
 #include <unity/scopes/ScopeExceptions.h>
 #include <unity/UnityExceptions.h>
+#include <unity/util/FileIO.h>
 
 #include <algorithm>
 #include <array>
@@ -30,10 +33,8 @@
 #include <fstream>
 #include <future>
 #include <utility>
-#include <iostream>
 #include <map>
 #include <sstream>
-#include <sys/stat.h>
 
 static std::string homedir()
 {
@@ -52,13 +53,17 @@ static const std::string c_preview_resource = "/preview";
 
 static const std::string c_scopes_cache_dir = homedir() + "/.cache/unity-scopes/";
 static const std::string c_scopes_cache_filename = "remote-scopes.json";
+static const std::string c_partner_id_file = "/custom/partner-id";
+
+// Some of the tests don't have a runtime, so we use a separate logger in that case.
+BOOST_LOG_INLINE_GLOBAL_LOGGER_DEFAULT(test_logger, boost::log::sources::severity_channel_logger_mt<>)
 
 using namespace unity::scopes;
 using namespace unity::scopes::internal::smartscopes;
 
 //-- SearchHandle
 
-SearchHandle::SearchHandle(uint search_id, SmartScopesClient::SPtr ssc)
+SearchHandle::SearchHandle(unsigned int search_id, SmartScopesClient::SPtr ssc)
     : search_id_(search_id)
     , ssc_(ssc)
 {
@@ -81,7 +86,7 @@ void SearchHandle::cancel_search()
 
 //-- PreviewHandle
 
-PreviewHandle::PreviewHandle(uint preview_id, SmartScopesClient::SPtr ssc)
+PreviewHandle::PreviewHandle(unsigned int preview_id, SmartScopesClient::SPtr ssc)
     : preview_id_(preview_id)
     , ssc_(ssc)
 {
@@ -106,12 +111,21 @@ void PreviewHandle::cancel_preview()
 
 SmartScopesClient::SmartScopesClient(HttpClientInterface::SPtr http_client,
                                      JsonNodeInterface::SPtr json_node,
-                                     std::string const& url)
+                                     RuntimeImpl* runtime,
+                                     std::string const& url,
+                                     std::string const& partner_id_path)
     : http_client_(http_client)
     , json_node_(json_node)
+    , logger_(runtime ? runtime->logger() : test_logger::get())
     , have_latest_cache_(false)
     , query_counter_(0)
+    , partner_file_(partner_id_path)
 {
+    if (partner_file_.size() == 0)
+    {
+        partner_file_ = c_partner_id_file;
+    }
+
     // initialise url_
     reset_url(url);
 
@@ -166,24 +180,46 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
             remote_scopes_uri << "locale=" << locale;
         }
 
-        std::cout << "SmartScopesClient.get_remote_scopes(): GET " << remote_scopes_uri.str() << std::endl;
+        BOOST_LOG_SEV(logger_, Logger::Info)
+            << "SmartScopesClient.get_remote_scopes(): GET " << remote_scopes_uri.str();
 
-        HttpResponseHandle::SPtr response = http_client_->get(remote_scopes_uri.str(), [&response_str](std::string const& replyLine) {
-                response_str += replyLine; // accumulate all reply lines
-        });
-        response->wait();
+        HttpHeaders headers;
+        try
+        {
+            auto partner_id = util::read_text_file(partner_file_);
+            if (!partner_id.empty())
+            {
+                std::string const user_agent_hdr = "partner=" + http_client_->to_percent_encoding(partner_id);
+                headers.push_back(std::make_pair("User-Agent", user_agent_hdr));
+                BOOST_LOG_SEV(logger_, Logger::Info) << "User agent: " << user_agent_hdr;
+            }
+        }
+        catch (std::exception const& e)
+        {
+            BOOST_LOG_SEV(logger_, Logger::Error) << "SmartScopesClient.get_remote_scopes(): failed to read " << partner_file_ << ": " << e.what();
+        }
 
-        std::cout << "SmartScopesClient.get_remote_scopes(): Remote scopes:" << std::endl << response_str << std::endl;
+        std::mutex reponse_mutex;
+        HttpResponseHandle::SPtr response = http_client_->get(remote_scopes_uri.str(), [&response_str, &reponse_mutex](std::string const& replyLine)
+        {
+            std::lock_guard<std::mutex> lock(reponse_mutex);
+            response_str += replyLine; // accumulate all reply lines
+        }, headers);
+
+        response->get();
+
+        BOOST_LOG_SEV(logger_, Logger::Info)
+            << "SmartScopesClient.get_remote_scopes(): Remote scopes:\n" << response_str;
     }
     catch (std::exception const& e)
     {
-        std::cerr << "SmartScopesClient.get_remote_scopes(): Failed to retrieve remote scopes from uri: "
-                  << url_ << c_remote_scopes_resource << std::endl;
+        BOOST_LOG(logger_)
+            << "SmartScopesClient.get_remote_scopes(): Failed to retrieve remote scopes from uri: "
+            << url_ << c_remote_scopes_resource << ": " << e.what();
 
         if (caching_enabled)
         {
-            std::cerr << e.what() << std::endl;
-            std::cerr << "SmartScopesClient.get_remote_scopes(): Using remote scopes from cache" << std::endl;
+            BOOST_LOG(logger_) << "SmartScopesClient.get_remote_scopes(): Using remote scopes from cache";
 
             response_str = read_cache();
             if (response_str.empty())
@@ -208,10 +244,10 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
         json_node_->read_json(response_str);
         root_node = json_node_->get_node();
     }
-    catch (std::exception const&)
+    catch (std::exception const& e)
     {
-        std::cerr << "SmartScopesClient.get_remote_scopes() Failed to parse json response from uri: "
-                  << url_ << c_remote_scopes_resource << std::endl;
+        BOOST_LOG(logger_) << "SmartScopesClient.get_remote_scopes() Failed to parse json response from uri: "
+                           << url_ << c_remote_scopes_resource << ": " << e.what();
         throw;
     }
 
@@ -226,7 +262,7 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
 
             if (!child_node->has_node("id"))
             {
-                std::cerr << "SmartScopesClient.get_remote_scopes(): Skipping scope with no id" << std::endl;
+                BOOST_LOG(logger_) << "SmartScopesClient.get_remote_scopes(): Skipping scope with no id";
                 continue;
             }
 
@@ -238,15 +274,15 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
             {
                 if (!child_node->has_node(field))
                 {
-                    std::cerr << "SmartScopesClient.get_remote_scopes(): Scope: \"" << scope.id
-                              << "\" has no \"" << field << "\" field" << std::endl;
+                    BOOST_LOG(logger_)
+                        << "SmartScopesClient.get_remote_scopes(): Scope: \"" << scope.id
+                        << "\" has no \"" << field << "\" field";
                     err = true;
                 }
             }
             if (err)
             {
-                std::cerr << "SmartScopesClient.get_remote_scopes(): Skipping scope: \""
-                          << scope.id << "\"" << std::endl;
+                BOOST_LOG(logger_) << "SmartScopesClient.get_remote_scopes(): Skipping scope: \"" << scope.id << "\"";
                 continue;
             }
 
@@ -282,20 +318,58 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
 
             scope.invisible = child_node->has_node("invisible") ? child_node->get_node("invisible")->as_bool() : false;
 
+            scope.version = child_node->has_node("version") ? child_node->get_node("version")->as_int() : 0;
+            if (scope.version < 0)
+            {
+                BOOST_LOG(logger_)
+                    << "SmartScopesClient.get_remote_scopes(): Scope: \"" << scope.id
+                    << "\" returned a negative \"version\" value, skipping scope";
+                continue;
+            }
+
+            if (child_node->has_node("keywords"))
+            {
+                auto node = child_node->get_node("keywords");
+                if (node->type() == JsonNodeInterface::NodeType::Array)
+                {
+                    auto keywords = node->to_variant().get_array();
+                    for (auto const& keyword : keywords)
+                    {
+                        try
+                        {
+                            scope.keywords.emplace(keyword.get_string());
+                        }
+                        catch (unity::LogicException const& e)
+                        {
+                            BOOST_LOG(logger_)
+                                << "SmartScopesClient.get_remote_scopes(): Scope: \"" << scope.id
+                                << "\" returned a non-string keyword";
+                        }
+                    }
+                }
+                else
+                {
+                    BOOST_LOG(logger_)
+                        << "SmartScopesClient.get_remote_scopes(): Scope: \"" << scope.id
+                        << "\" returned an invalid value type for \"keywords\"";
+                }
+            }
+
             remote_scopes.push_back(scope);
         }
         catch (std::exception const& e)
         {
-            std::cerr << "SmartScopesClient.get_remote_scopes(): Skipping scope: \""
-                      << scope.id << "\" due to a json parsing failure" << std::endl;
-            std::cerr << e.what() << std::endl;
+            BOOST_LOG(logger_)
+                << "SmartScopesClient.get_remote_scopes(): Skipping scope: \""
+                << scope.id << "\" due to a json parsing failure: " << e.what();
         }
     }
 
     if (remote_scopes.empty())
     {
-        std::cerr << "SmartScopesClient.get_remote_scopes(): No valid remote scopes retrieved from uri: "
-                  << url_ << c_remote_scopes_resource << std::endl;
+        BOOST_LOG(logger_)
+            << "SmartScopesClient.get_remote_scopes(): No valid remote scopes retrieved from uri: "
+            << url_ << c_remote_scopes_resource;
     }
     else if (!using_cache)
     {
@@ -307,18 +381,20 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
             }
             catch (std::exception const& e)
             {
-                std::cerr << "SmartScopesClient.get_remote_scopes(): Failed to write to cache file: "
-                          << c_scopes_cache_dir << c_scopes_cache_filename << std::endl;
-                std::cerr << e.what() << std::endl;
+                BOOST_LOG(logger_)
+                    << "SmartScopesClient.get_remote_scopes(): Failed to write to cache file: "
+                    << c_scopes_cache_dir << c_scopes_cache_filename << ": " << e.what();
             }
         }
 
-        std::cout << "SmartScopesClient.get_remote_scopes(): Retrieved remote scopes from uri: "
-                  << url_ << c_remote_scopes_resource << std::endl;
+        BOOST_LOG_SEV(logger_, Logger::Info)
+            << "SmartScopesClient.get_remote_scopes(): Retrieved remote scopes from uri: "
+            << url_ << c_remote_scopes_resource;
     }
     else
     {
-        std::cout << "SmartScopesClient.get_remote_scopes(): Retrieved remote scopes from cache" << std::endl;
+        BOOST_LOG_SEV(logger_, Logger::Info)
+            << "SmartScopesClient.get_remote_scopes(): Retrieved remote scopes from cache";
     }
 
     return !using_cache;
@@ -334,11 +410,13 @@ SearchHandle::UPtr SmartScopesClient::search(SearchReplyHandler const& handler,
                                              VariantMap const& settings,
                                              VariantMap const& filter_state,
                                              std::string const& locale,
-                                             std::string const& country,
+                                             LocationInfo const& location,
                                              std::string const& user_agent_hdr,
-                                             uint limit)
+                                             unsigned int limit)
 {
     std::ostringstream search_uri;
+    search_uri.imbue(std::locale::classic()); // so that doubles use one standard formatting wrt decimal point
+
     search_uri << base_url << c_search_resource << "?";
 
     // mandatory parameters
@@ -365,9 +443,13 @@ SearchHandle::UPtr SmartScopesClient::search(SearchReplyHandler const& handler,
     {
         search_uri << "&locale=" << locale;
     }
-    if (!country.empty())
+    if (location.has_location)
     {
-        search_uri << "&country=" << country;
+        if (!location.country_code.empty())
+        {
+            search_uri << "&country=" << location.country_code;
+        }
+        search_uri << std::fixed << std::setprecision(5) << "&latitude=" << location.latitude << "&longitude=" << location.longitude;
     }
     if (limit != 0)
     {
@@ -379,26 +461,28 @@ SearchHandle::UPtr SmartScopesClient::search(SearchReplyHandler const& handler,
     }
 
     std::lock_guard<std::mutex> lock(query_results_mutex_);
-    uint search_id = ++query_counter_;
+    unsigned int search_id = ++query_counter_;
 
-    std::cout << "SmartScopesClient.search(): GET " << search_uri.str() << std::endl;
+    BOOST_LOG_SEV(logger_, Logger::Info) << "SmartScopesClient.search(): GET " << search_uri.str();
 
     HttpHeaders headers;
     if (!user_agent_hdr.empty())
     {
-        std::cout << "User agent: " << user_agent_hdr;
+        BOOST_LOG_SEV(logger_, Logger::Info) << "User agent: " << user_agent_hdr;
         headers.push_back(std::make_pair("User-Agent", user_agent_hdr));
     }
 
-    query_results_[search_id] = http_client_->get(search_uri.str(), [this, handler](std::string const& lineData) {
-            try
-            {
-                parse_line(lineData, handler);
-            }
-            catch (std::exception const &e)
-            {
-                std::cerr << "Failed to parse: " << e.what() << std::endl;
-            }
+    auto tmp_data = std::make_shared<std::string>();
+    query_results_[search_id] = http_client_->get(search_uri.str(), [this, tmp_data, handler](std::string const& chunk)
+    {
+        // prepend any leftover data from the previous handle_chunk call
+        std::string data = *tmp_data + (chunk.empty() ? "\n" : chunk);
+
+        // store the leftover data from the handle_chunk call into tmp_data
+        *tmp_data = handle_chunk(data, [this, handler](const std::string& line)
+        {
+            handle_line(line, handler);
+        });
     }, headers);
 
     return SearchHandle::UPtr(new SearchHandle(search_id, shared_from_this()));
@@ -409,11 +493,11 @@ PreviewHandle::UPtr SmartScopesClient::preview(PreviewReplyHandler const& handle
                                                std::string const& result,
                                                std::string const& session_id,
                                                std::string const& platform,
-                                               const uint widgets_api_version,
+                                               const unsigned int widgets_api_version,
                                                VariantMap const& settings,
                                                std::string const& locale,
-                                               std::string const& user_agent_hdr,
-                                               std::string const& country)
+                                               std::string const& country,
+                                               std::string const& user_agent_hdr)
 {
     std::ostringstream preview_uri;
     preview_uri << base_url << c_preview_resource << "?";
@@ -451,24 +535,58 @@ PreviewHandle::UPtr SmartScopesClient::preview(PreviewReplyHandler const& handle
     }
 
     std::lock_guard<std::mutex> lock(query_results_mutex_);
-    uint preview_id = ++query_counter_;
+    unsigned int preview_id = ++query_counter_;
 
-    std::cout << "SmartScopesClient.preview(): GET " << preview_uri.str() << std::endl;
-    query_results_[preview_id] = http_client_->get(preview_uri.str(), [this, handler](std::string const& lineData) {
-            try
-            {
-                parse_line(lineData, handler);
-            }
-            catch (std::exception const &e)
-            {
-                std::cerr << "Failed to parse: " << e.what() << std::endl;
-            }
+    BOOST_LOG_SEV(logger_, Logger::Info) << "SmartScopesClient.preview(): GET " << preview_uri.str();
+
+    auto tmp_data = std::make_shared<std::string>();
+    query_results_[preview_id] = http_client_->get(preview_uri.str(), [this, tmp_data, handler](std::string const& chunk)
+    {
+        // prepend any leftover data from the previous handle_chunk call
+        std::string data = *tmp_data + (chunk.empty() ? "\n" : chunk);
+
+        // store the leftover data from the handle_chunk call into tmp_data
+        *tmp_data = handle_chunk(data, [this, handler](const std::string& line)
+        {
+            handle_line(line, handler);
+        });
     }, headers);
 
     return PreviewHandle::UPtr(new PreviewHandle(preview_id, shared_from_this()));
 }
 
-void SmartScopesClient::parse_line(std::string const& json, PreviewReplyHandler const& handler)
+std::string SmartScopesClient::handle_chunk(const std::string& chunk, std::function<void(const std::string&)> line_handler)
+{
+    // According to the docs, we expect:
+    // The response will have Content-Type
+    // application/json, it will be a chunked response, in practice a series of
+    // “\r\n” delimited lines, each containing one JSON object, with the
+    // possible forms, matching what currently can be pushed into a reply in the
+    // new scopes API
+    static constexpr const char separator{'\n'};
+
+    // read data line-by-line calling line_handler() for each
+    auto newline_pos = 0;
+    auto endline_pos = chunk.find(separator);
+    while (endline_pos != std::string::npos)
+    {
+        try
+        {
+            line_handler(chunk.substr(newline_pos, endline_pos - newline_pos));
+        }
+        catch (std::exception const &e)
+        {
+            BOOST_LOG_SEV(logger_, Logger::Error) << "SmartScopesClient.handle_chunk(): Failed to parse line: " << e.what();
+        }
+        newline_pos = endline_pos + 1;
+        endline_pos = chunk.find(separator, newline_pos);
+    }
+
+    // return the leftover data
+    return chunk.substr(newline_pos, chunk.size() - newline_pos);
+}
+
+void SmartScopesClient::handle_line(std::string const& json, PreviewReplyHandler const& handler)
 {
     JsonNodeInterface::SPtr root_node;
     JsonNodeInterface::SPtr child_node;
@@ -516,7 +634,7 @@ void SmartScopesClient::parse_line(std::string const& json, PreviewReplyHandler
     }
 }
 
-void SmartScopesClient::parse_line(std::string const& json, SearchReplyHandler const& handler)
+void SmartScopesClient::handle_line(std::string const& json, SearchReplyHandler const& handler)
 {
     JsonNodeInterface::SPtr root_node;
     JsonNodeInterface::SPtr child_node;
@@ -596,7 +714,7 @@ void SmartScopesClient::parse_line(std::string const& json, SearchReplyHandler c
     }
 }
 
-void SmartScopesClient::wait_for_search(uint search_id)
+void SmartScopesClient::wait_for_search(unsigned int search_id)
 {
     try
     {
@@ -618,7 +736,9 @@ void SmartScopesClient::wait_for_search(uint search_id)
     }
     catch (std::exception const& e)
     {
-        std::cerr << "SmartScopesClient.get_search_results(): Failed to retrieve search results for query " << search_id << ": " << e.what() << std::endl;
+        BOOST_LOG(logger_)
+            << "SmartScopesClient.get_search_results(): Failed to retrieve search results for query "
+            << search_id << ": " << e.what();
         throw;
     }
 
@@ -665,13 +785,15 @@ std::shared_ptr<DepartmentInfo> SmartScopesClient::parse_departments(JsonNodeInt
             catch (std::exception const& e)
             {
                 // error in one subdepartment is not critical - just ignore it
-                std::cerr << "SmartScopesClient::parse_departments(): Error parsing subdepartment of department '" << dep->label << "': " << e.what() << std::endl;
+                BOOST_LOG(logger_)
+                    << "SmartScopesClient::parse_departments(): Error parsing subdepartment of department '"
+                    << dep->label << "': " << e.what();
             }
         }
         if(subdeps->size() > 0 && dep->subdepartments.size() == 0)
         {
             std::stringstream err;
-            err << "SmartScopesClient::parse_departments(): Failed to parse subdepartments of department '" << dep->label;
+            err << "SmartScopesClient::parse_departments(): Failed to parse subdepartments of department '" << dep->label << "'";
             throw LogicException(err.str());
         }
     }
@@ -698,7 +820,7 @@ FilterState SmartScopesClient::parse_filter_state(JsonNodeInterface::SPtr node)
     return FilterStateImpl::deserialize(node->to_variant().get_dict());
 }
 
-void SmartScopesClient::wait_for_preview(uint preview_id)
+void SmartScopesClient::wait_for_preview(unsigned int preview_id)
 {
     try
     {
@@ -720,7 +842,9 @@ void SmartScopesClient::wait_for_preview(uint preview_id)
     }
     catch (std::exception const& e)
     {
-        std::cerr << "SmartScopesClient.get_preview_results(): Failed to retrieve preview results for query " << preview_id << std::endl;
+        BOOST_LOG(logger_)
+            << "SmartScopesClient.get_preview_results(): Failed to retrieve preview results for query "
+            << preview_id;
         throw;
     }
 
@@ -732,7 +856,7 @@ std::vector<std::string> SmartScopesClient::extract_json_stream(std::string cons
 {
     std::vector<std::string> jsons;
 
-    uint start_pos = 0;
+    unsigned int start_pos = 0;
 
     while (start_pos < json_stream.size())
     {
@@ -750,7 +874,7 @@ std::vector<std::string> SmartScopesClient::extract_json_stream(std::string cons
     return jsons;
 }
 
-void SmartScopesClient::cancel_query(uint query_id)
+void SmartScopesClient::cancel_query(unsigned int query_id)
 {
     std::lock_guard<std::mutex> lock(query_results_mutex_);
 
@@ -765,7 +889,7 @@ void SmartScopesClient::cancel_query(uint query_id)
 void SmartScopesClient::write_cache(std::string const& scopes_json)
 {
     // make cache directory (fails silently if already exists)
-    mkdir(c_scopes_cache_dir.c_str(), 0755);
+    make_directories(c_scopes_cache_dir, 0755);
 
     // open cache for output
     std::ofstream cache_file(c_scopes_cache_dir + c_scopes_cache_filename);
@@ -834,7 +958,9 @@ std::string SmartScopesClient::stringify_settings(VariantMap const& settings)
             setting_valid = true;
             break;
         default:
-            std::cerr << "SmartScopesClient.stringify_settings(): Ignoring unsupported Variant type for settings value: \"" << setting.first << "\"" << std::endl;
+            BOOST_LOG(logger_)
+                << "SmartScopesClient.stringify_settings(): Ignoring unsupported Variant type for settings value: \""
+                << setting.first << "\"";
         }
 
         // If we have constructed a valid id:value string, append it to result_str
@@ -853,3 +979,8 @@ std::string SmartScopesClient::stringify_settings(VariantMap const& settings)
     }
     return result_str.str();
 }
+
+boost::log::sources::severity_channel_logger_mt<>& SmartScopesClient::logger() const
+{
+    return logger_;
+}
